
# THIS FILE IS GENERATED FROM SCIKIT-FUSION SETUP.PY
short_version = '0.2.1'
version = '0.2.1'
full_version = '0.2.1.dev0+0159eb8'
git_revision = '0159eb86a47f277072ea4cf1d30798e2f751f7d4'
release = False

if not release:
    version = full_version
    short_version += ".dev"
